import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { useEffect, useState } from 'react';
import { Controller, useForm } from 'react-hook-form';
import { z } from 'zod';
// import { DevTool } from '@hookform/devtools';
import { zodResolver } from '@hookform/resolvers/zod';
import AdminToggle from '../../components/toggle/AdminToggle';
import AdminUploader from '../../components/uploader/AdminUploader';
import { ButtonVariant, EditApiMethod, FieldEnum } from '../../constants/common-constants';
import { InputPresets } from '../../constants/input-field-constants';
import { translate } from '../../locales/translations';
import AdminButton from '../button/AdminButton';
import AdminCheckbox from '../checkbox/AdminCheckbox';
import AdminChipInput from '../chip-input/AdminChipInput';
import AdminDatePicker from '../date-time-picker/AdminDatePicker';
import AdminDateTimePicker from '../date-time-picker/AdminDateTimePicker';
import AdminTimePicker from '../date-time-picker/AdminTimePicker';
import AdminDateRangePicker from '../date-time-range-picker/DateRangePicker';
import AdminTimeRangePicker from '../date-time-range-picker/TimeRangePicker';
import AdminDropdown from '../dropdown/AdminDropdown';
import AdminInput from '../input/AdminInput';
import AdminRadio from '../radio/AdminRadio';
import AdminSideDrawer from '../side-drawer/AdminSideDrawer';
import AdminSlider from '../slider/AdminSlider';
import AdminRepeater from './AdminRepeater';
import './adminForm-styles.scss';
export var FormTypeEnum;
(function (FormTypeEnum) {
    FormTypeEnum["Add"] = "Add";
    FormTypeEnum["Edit"] = "Edit";
    FormTypeEnum["Filter"] = "Filter";
})(FormTypeEnum || (FormTypeEnum = {}));
const FILTER_SCHEMA = z.any(); // we not putting any validation in case of filters
function Form(props) {
    const { schema, defaultValues, fields, submitTitle = translate.components.form.addEdit.submit, onSubmit, onSecondaryButtonClick = () => { }, secondaryButtonTitle = '', type, isLoading, editApiMethod, } = props;
    const [isSubmitDisabled, setIsSubmitDisabled] = useState(true);
    const methods = useForm({
        resolver: zodResolver(type === FormTypeEnum.Filter ? FILTER_SCHEMA : schema),
        defaultValues,
        mode: 'onSubmit',
    });
    const { handleSubmit, control, formState: { errors, isValid }, watch, } = methods;
    const watchAllFields = watch();
    useEffect(() => {
        if (type === FormTypeEnum.Edit) {
            const hasChanges = Object.keys(watchAllFields).some(key => JSON.stringify(watchAllFields[key]) !== JSON.stringify(defaultValues[key]));
            setIsSubmitDisabled(!hasChanges);
        }
        else {
            setIsSubmitDisabled(false);
        }
    }, [watchAllFields, defaultValues, type, isValid]);
    const onSubmitHandler = (data) => {
        if (type === FormTypeEnum.Edit && editApiMethod === EditApiMethod.PATCH) {
            const changedData = Object.keys(data).reduce((acc, key) => {
                if (JSON.stringify(data[key]) !== JSON.stringify(defaultValues[key])) {
                    acc[key] = data[key];
                }
                return acc;
            }, {});
            if (Object.keys(changedData).length > 0) {
                onSubmit(type)(changedData);
            }
        }
        else {
            onSubmit(type)(data);
        }
    };
    const renderField = (renderProps) => {
        const { fieldProps, filterProps, headerName, field: fieldId, errorMessage } = renderProps;
        const finalFieldProps = type === FormTypeEnum.Filter ? filterProps : fieldProps;
        const { fieldType, ...restFieldProps } = finalFieldProps || {};
        const error = errors[fieldId];
        const helperText = error?.message;
        const commonProps = {
            label: headerName,
            id: fieldId,
            helperText: errorMessage || helperText,
        };
        let Component = null;
        switch (fieldType) {
            case FieldEnum.Radio:
                Component = (_jsx(Controller, { control: control, name: fieldId, render: ({ field, fieldState: { error } }) => {
                        const { ref, ...restField } = field;
                        return (_jsx(AdminRadio, { error: !!error, ...commonProps, ...restField, ...restFieldProps }));
                    } }, fieldId));
                break;
            case FieldEnum.Text:
                Component = (_jsx(Controller, { control: control, name: fieldId, render: ({ field, fieldState: { error } }) => {
                        const { ref, ...restField } = field;
                        return (_jsx(AdminInput, { error: !!error, ...commonProps, ...restField, ...restFieldProps }));
                    } }, fieldId));
                break;
            case FieldEnum.ChipInput:
                Component = (_jsx(Controller, { control: control, name: fieldId, render: ({ field, fieldState: { error } }) => {
                        const { ref, ...restField } = field;
                        return (_jsx(AdminChipInput, { error: !!error, ...commonProps, ...restField, ...restFieldProps }));
                    } }, fieldId));
                break;
            case FieldEnum.Number:
                Component = (_jsx(Controller, { control: control, name: fieldId, render: ({ field, fieldState: { error } }) => {
                        const { ref, ...restField } = field;
                        return (_jsx(AdminInput, { error: !!error, preset: InputPresets.Number, ...commonProps, ...restField, ...restFieldProps }));
                    } }, fieldId));
                break;
            case FieldEnum.Checkbox:
                Component = (_jsx(Controller, { control: control, name: fieldId, render: ({ field, fieldState: { error } }) => {
                        const { ref, ...restField } = field;
                        return (_jsx(AdminCheckbox, { error: !!error, ...commonProps, ...restField, ...restFieldProps }));
                    } }, fieldId));
                break;
            case FieldEnum.Dropdown:
                Component = (_jsx(Controller, { control: control, name: fieldId, render: ({ field, fieldState: { error } }) => {
                        const { ref, ...restField } = field;
                        return (_jsx(AdminDropdown, { error: !!error, ...commonProps, ...restField, ...restFieldProps }));
                    } }, fieldId));
                break;
            case FieldEnum.Date:
                Component = (_jsx(Controller, { control: control, name: fieldId, render: ({ field, fieldState: { error } }) => {
                        const { ref, ...restField } = field;
                        return (_jsx(AdminDatePicker, { error: !!error, ...commonProps, ...restField, ...restFieldProps }));
                    } }, fieldId));
                break;
            case FieldEnum.Time:
                Component = (_jsx(Controller, { control: control, name: fieldId, render: ({ field, fieldState: { error } }) => {
                        const { ref, ...restField } = field;
                        return (_jsx(AdminTimePicker, { error: !!error, ...commonProps, ...restField, ...restFieldProps }));
                    } }, fieldId));
                break;
            case FieldEnum.DateRange:
                Component = (_jsx(Controller, { control: control, name: fieldId, render: ({ field, fieldState: { error } }) => {
                        const { ref, ...restField } = field;
                        return (_jsx(AdminDateRangePicker, { errors: error, ...commonProps, ...restField, ...restFieldProps }));
                    } }, fieldId));
                break;
            case FieldEnum.TimeRange:
                Component = (_jsx(Controller, { control: control, name: fieldId, render: ({ field, fieldState: { error } }) => {
                        const { ref, ...restField } = field;
                        return (_jsx(AdminTimeRangePicker, { ...commonProps, ...restField, ...restFieldProps, errors: error || {} }));
                    } }, fieldId));
                break;
            case FieldEnum.Slider:
                Component = (_jsx(Controller, { control: control, name: fieldId, render: ({ field, fieldState: { error } }) => {
                        const { ref, ...restField } = field;
                        return (_jsx(AdminSlider, { ...commonProps, ...restField, ...restFieldProps, error: !!error }));
                    } }, fieldId));
                break;
            case FieldEnum.Toggle:
                Component = (_jsx(Controller, { control: control, name: fieldId, render: ({ field }) => {
                        const { ref, ...restField } = field;
                        return (_jsx(AdminToggle, { ...commonProps, ...restField, ...restFieldProps }));
                    } }, fieldId));
                break;
            case FieldEnum.REPEATER:
                const { columns: repeaterFields } = finalFieldProps;
                Component = (_jsx(AdminRepeater, { control: control, errors: errors, name: fieldId, repeaterFields: repeaterFields, renderField: renderField, ...restFieldProps, ...commonProps }, fieldId));
                break;
            case FieldEnum.DateTime:
                Component = (_jsx(Controller, { control: control, name: fieldId, render: ({ field, fieldState: { error } }) => {
                        const { ref, ...restField } = field;
                        return (_jsx(AdminDateTimePicker, { error: !!error, ...commonProps, ...restField, ...restFieldProps }));
                    } }, fieldId));
                break;
            case FieldEnum.UPLOADER:
                Component = (_jsx(Controller, { control: control, name: fieldId, render: ({ field, fieldState: { error } }) => {
                        const { ref, ...restField } = field;
                        return (_jsx(AdminUploader, { error: !!error, ...commonProps, ...restField, ...restFieldProps }));
                    } }, fieldId));
                break;
            default:
                Component = null;
        }
        return Component;
    };
    return (_jsx(_Fragment, { children: _jsxs("form", { onSubmit: handleSubmit(onSubmitHandler), className: "addEdit__form", children: [_jsx("div", { className: "addEdit__formWrapper", children: fields.map(renderField) }), _jsxs("div", { className: "addEdit__footer", children: [secondaryButtonTitle && (_jsx(AdminButton, { handleClick: onSecondaryButtonClick, title: secondaryButtonTitle, isFullWidth: false, buttonVariant: ButtonVariant.Text })), _jsx(AdminButton, { type: "submit", title: submitTitle, isFullWidth: false, disabled: isSubmitDisabled, className: "addEdit__submit", isLoading: isLoading })] })] }) }));
}
function AdminForm(props) {
    const { onClose, open, title, type = FormTypeEnum.Add, ...rest } = props;
    return (_jsx(AdminSideDrawer, { open: open, onClose: onClose, title: `${type ?? ''} ${title}`, children: _jsx(Form, { ...rest, type: type }) }));
}
export default AdminForm;
