import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { useCallback, useEffect, useState } from 'react';
import { useLocation, useNavigate, useSearchParams } from 'react-router-dom';
import { cloneDeep, debounce, isNil, omitBy } from 'lodash';
import { observer } from 'mobx-react-lite';
import qs from 'qs';
import AddEditForm, { FormTypeEnum } from '../../components/form/AdminForm';
import ConfirmationModal from '../../components/modal/confirmation-modal/ConfirmationModal';
import AdminTab from '../../components/tab/AdminTab';
import { ConfirmationModalPreset, EditApiMethod, TABLE_ACTIONS_KEYS, TableSortingOrder, ToastPreset, } from '../../constants';
import { useVsAdmin } from '../../context/VsAdminProvider';
import { useTableData } from '../../hooks/useData';
import { translate } from '../../locales/translations';
import { useStore } from '../../models';
import { PaginationMethod, } from '../../types';
import { removeKeyFromQueryString } from '../../utils/common-utils';
import './withCrud-styles.scss';
const WithOps = observer((props) => {
    const { Component, data, title, handleCustomRowClick } = props;
    const { key, columns, defaultValues, schema, formatData, apiEndPoint, detailsRoute, paginationMethod = PaginationMethod.PAGE, handleDetailsSubmit, customActions, pagination, editApiMethod = EditApiMethod.PATCH, actionDisableForRow, ...rest } = data;
    const store = useStore();
    const { viewStore } = store;
    const { toastStore } = viewStore;
    const { popToast } = toastStore;
    const { getToken } = useVsAdmin();
    const [searchParams, setSearchParams] = useSearchParams();
    const location = useLocation();
    const navigation = useNavigate();
    const { pathname } = location;
    const page = searchParams.get('page') || 1;
    const limit = searchParams.get('limit') || 10;
    const sortBy = searchParams.get('sortBy') || '';
    const order = searchParams.get('order') || '';
    const searchItem = searchParams.get('search_term') || '';
    const { filters } = omitBy(cloneDeep(qs.parse(location.search, { ignoreQueryPrefix: true })), isNil) || {};
    const filterParams = qs.stringify({ filters: filters }, { encode: false });
    const params = {
        getToken,
        key,
        page: +page,
        limit: +limit,
        sortBy,
        order,
        filters,
        apiEndPoint,
        search_term: searchItem,
        paginationMethod,
        pagination,
        handleDetailsSubmit,
        editApiMethod,
    };
    const { data: apiData, isLoading, deleteItem, isPending, addItem, updateItem, refetch, updateStatus, } = useTableData(params);
    const { details, totalRecords } = apiData || {};
    const [rows, setRows] = useState([]);
    const [isDeleteModalOpen, setIsDeleteModalOpen] = useState(false);
    const [searchTerm, setSearchTerm] = useState(searchItem);
    const [formOptions, setFormOptions] = useState({
        isOpen: false,
        type: null,
    });
    const [currentRow, setCurrentRow] = useState({});
    useEffect(() => {
        setRows(details);
    }, [apiData, page, limit, sortBy, order, searchItem]);
    useEffect(() => {
        if (!!totalRecords) {
            setSearchParams(prevParams => {
                const newParams = new URLSearchParams(prevParams);
                newParams.set('totalRecords', String(totalRecords));
                return newParams;
            });
        }
    }, [totalRecords, setSearchParams]);
    useEffect(() => {
        setSearchParams(prevParams => {
            const newParams = new URLSearchParams(prevParams);
            newParams.set('title', title);
            return newParams;
        });
    }, [title, setSearchParams]);
    // The handleActionClick function is used to handle the action click event.
    const handleActionClick = (data, action) => {
        switch (action) {
            case TABLE_ACTIONS_KEYS.EDIT:
                setFormOptions({ isOpen: true, type: FormTypeEnum.Edit });
                setCurrentRow(() => data.row);
                break;
            case TABLE_ACTIONS_KEYS.DELETE:
                setIsDeleteModalOpen(() => true);
                setCurrentRow(() => data.row);
                break;
            case TABLE_ACTIONS_KEYS.VIEW:
                handleRowClick(data);
                break;
            default:
                break;
        }
    };
    const handleDeleteSuccess = () => {
        refetch();
        popToast({
            type: ToastPreset.Success,
            message: 'Record deleted successfully!',
        });
    };
    const handleDeleteClick = () => {
        const { id } = currentRow || {};
        id && deleteItem(id, handleDeleteSuccess, onError);
        setIsDeleteModalOpen(() => false);
    };
    const handleCloseDeleteModal = () => {
        setIsDeleteModalOpen(false);
    };
    //  The handlePageChange function is used to navigate to the next or previous page.
    const handlePageChange = (page) => {
        navigation(`${pathname}?page=${page}&limit=${limit}&sortBy=${sortBy}&search_term=${searchItem}&order=${order}&${filterParams}&totalRecords=${totalRecords}`);
    };
    //  The handleRowsPerPageChange function is used to change limit of rows per page.
    const handleRowsPerPageChange = (rowsPerPage) => {
        navigation(`${pathname}?page=${1}&limit=${rowsPerPage}&sortBy=${sortBy}&search_term=${searchItem}&order=${order}&${filterParams}`);
    };
    //  The handleSort function is used to sort the data based on the field and sort order.
    const handleSort = (model) => {
        const { field, sort = TableSortingOrder.ASC } = model[0];
        const sortType = sort === 'asc' ? TableSortingOrder.ASC : TableSortingOrder.DESC;
        navigation(`${pathname}?page=${page}&limit=${limit}&sortBy=${field}&order=${sortType}&search_term=${searchItem}&${filterParams}`);
    };
    //  The handleAddClick function is used to open to the add page
    const handleAddClick = () => {
        setFormOptions({ isOpen: true, type: FormTypeEnum.Add });
    };
    //  The handleFormClose function is used to close to the form page
    const handleFormClose = () => {
        setFormOptions({ isOpen: false, type: null });
    };
    const handleFilterButtonClick = () => {
        setFormOptions({ isOpen: true, type: FormTypeEnum.Filter });
    };
    const handleFilterReset = () => {
        const newParams = removeKeyFromQueryString(location.search, 'filters');
        navigation(`${pathname}${newParams}`);
        setFormOptions({ isOpen: false, type: null });
    };
    const debouncedSearch = useCallback(debounce((value) => {
        navigation(`${pathname}?page=${page}&limit=${limit}&sortBy=${sortBy}&search_term=${value}&order=${order}&${filterParams}`);
    }, 300), [page, limit, sortBy, order, filterParams, pathname]);
    const handleSearch = (value) => {
        setSearchTerm(value);
        debouncedSearch(value);
    };
    const afterAddSuccess = () => {
        refetch();
        popToast({
            type: ToastPreset.Success,
            message: 'Record added successfully!',
        });
        handleFormClose();
    };
    const afterUpdateSuccess = () => {
        refetch();
        popToast({
            type: ToastPreset.Success,
            message: 'Record updated successfully!',
        });
        handleFormClose();
    };
    const onError = () => {
        popToast({
            type: ToastPreset.Error,
            message: 'Something went wrong!',
        });
        handleFormClose();
    };
    const handleSubmit = formType => (data, _event) => {
        const { id } = currentRow || {};
        switch (formType) {
            case FormTypeEnum.Add:
                addItem(data, afterAddSuccess, onError);
                break;
            case FormTypeEnum.Edit:
                if (id)
                    updateItem(id, data, afterUpdateSuccess, onError);
                break;
            case FormTypeEnum.Filter:
                const filtersParams = qs.stringify({ filters: data }, { encode: false });
                handleFormClose();
                navigation(`${pathname}?page=${1}&limit=${limit}&sortBy=${sortBy}&order=${order}&${filtersParams}&title=${title}&totalRecords=${totalRecords}`);
                break;
            default:
                break;
        }
    };
    const addEditColumns = columns?.filter(item => !!item.fieldProps);
    const filterColumns = columns?.filter(item => !!item.fieldProps || !!item.filterProps);
    const selectedRowData = (currentRow ? currentRow : defaultValues);
    let prefillData = defaultValues;
    switch (formOptions.type) {
        case FormTypeEnum.Edit:
            prefillData = selectedRowData;
            break;
        case FormTypeEnum.Filter:
            prefillData = filters;
            break;
        default:
            break;
    }
    const handleRowClick = (data) => {
        const myTitle = `${title} Details / ${data.row.id}`;
        handleCustomRowClick
            ? handleCustomRowClick(data)
            : navigation(`${detailsRoute}?id=${data.row.id}&title=${myTitle}
      `, {
                state: {
                    rowData: data.row,
                },
            });
    };
    const updateStatusSuccess = () => {
        refetch();
        popToast({
            type: ToastPreset.Success,
            message: 'Record updated successfully!',
        });
    };
    const handleToggleAction = (row) => {
        const { id = '' } = row;
        updateStatus(id, updateStatusSuccess, onError);
    };
    const disableActionsForRow = (row) => {
        return actionDisableForRow ? actionDisableForRow(row) : false;
    };
    return (_jsxs(_Fragment, { children: [_jsx(Component, { ...props, currentPage: +page, handleActionClick: handleActionClick, handleAddClick: handleAddClick, handleFilter: handleFilterButtonClick, handlePageChange: handlePageChange, handleRowClick: handleRowClick, handleRowsPerPageChange: handleRowsPerPageChange, handleSort: handleSort, isLoading: isLoading, pageSize: +limit, handleSearch: handleSearch, rows: rows, totalRecords: totalRecords, appliedFilters: filters, searchValue: searchTerm, handleToggle: handleToggleAction, disableActionsForRow: disableActionsForRow, ...rest }), _jsx(ConfirmationModal, { subHeaderTitle: 'Are you sure you want to delete this record?', open: isDeleteModalOpen, onButtonClick: handleDeleteClick, onClose: handleCloseDeleteModal, modalType: ConfirmationModalPreset.Delete }), _jsx(AddEditForm, { onClose: handleFormClose, open: formOptions.isOpen &&
                    (formOptions.type === FormTypeEnum.Add ||
                        formOptions.type === FormTypeEnum.Edit ||
                        formOptions.type === FormTypeEnum.Filter), fields: formOptions.type === FormTypeEnum.Filter ? filterColumns : addEditColumns, schema: schema, defaultValues: prefillData, type: formOptions.type, onSubmit: handleSubmit, title: title, secondaryButtonTitle: formOptions.type === FormTypeEnum.Filter
                    ? translate.components.form.addEdit.reset
                    : translate.components.form.addEdit.cancel, onSecondaryButtonClick: formOptions.type === FormTypeEnum.Filter ? handleFilterReset : handleFormClose, isLoading: isPending, editApiMethod: editApiMethod })] }));
});
function WithCrud(Component) {
    return function WithCrud(props) {
        const { data, handleCustomRowClick } = props;
        const [searchParams] = useSearchParams();
        const { tables, title } = data;
        const isTabView = tables.length > 1;
        searchParams.get('totalRecords') || '0';
        const tabData = tables.map(eachTable => {
            return {
                id: eachTable.key,
                name: eachTable?.subTitle || '',
                path: eachTable.key,
                children: (_jsx(WithOps, { Component: Component, data: eachTable, title: title }, eachTable.key)),
            };
        });
        return (_jsx(_Fragment, { children: isTabView ? (_jsx(AdminTab, { data: tabData })) : (_jsx(WithOps, { Component: Component, data: tables[0], title: title, handleCustomRowClick: handleCustomRowClick })) }));
    };
}
export { WithCrud };
