import { useMutation, useQuery, useQueryClient } from '@tanstack/react-query';
import { EditApiMethod } from '../constants';
import { HttpVerb, makeApiCall } from '../services';
import { PaginationMethod } from '../types/common-types';
/*
 * Function to get the data of a table
 * @param endPoint: string
 * @param page: number
 * @param limit: number
 * @returns {Promise}
 */
const getTableData = async (getToken, _, page, limit, sortBy = '', order = '', filters, apiURL, search_term = '', paginationMethod, pagination) => {
    const params = {
        ...(paginationMethod === PaginationMethod.PAGE && pagination && page && { page }),
        ...(paginationMethod === PaginationMethod.OFFSET && pagination && page && { offset: page - 1 }),
        ...(limit && { limit }),
        ...(search_term && { search_term }),
        ...(sortBy && { sortBy }),
        ...(!!order && { order }),
        ...filters,
    };
    const result = await makeApiCall(HttpVerb.GET, apiURL, undefined, {
        params,
    }, '', getToken);
    return {
        details: result?.data?.data ?? [],
        totalRecords: result?.data?.total_count ?? 0,
    };
};
/*
 * Function to delete an item
 * @param id: number | string
 * @param apiURL: string
 */
const deleteItem = async (id, apiURL, getToken) => {
    const result = await makeApiCall(HttpVerb.DELETE, `${apiURL}/${id}`, undefined, undefined, '', getToken);
    return result;
};
/*
 * Function to add an item
 * @param data: any
 * @param apiURL: string
 */
const addItem = async (data, apiURL, getToken) => {
    const result = await makeApiCall(HttpVerb.POST, apiURL, data, undefined, '', getToken);
    return result;
};
/*
 * Function to update an item
 * @param id: number | string
 * @param data: any
 * @param apiURL: string
 */
const updateItem = async (id, data, apiURL, method, getToken) => {
    const result = await makeApiCall(method, `${apiURL}/${id}`, data, undefined, '', getToken);
    return result;
};
const updateStatus = async (id, apiURL, getToken) => {
    const result = await makeApiCall(HttpVerb.PATCH, `${apiURL}/${id}/activate`, undefined, undefined, '', getToken);
    return result;
};
const useTableData = (props) => {
    const { getToken = () => '', key = '', page = 1, limit = 10, sortBy = '', order = '', filters = {}, apiEndPoint = {}, search_term = '', paginationMethod = PaginationMethod.PAGE, pagination = true, handleDetailsSubmit, editApiMethod = EditApiMethod.PATCH, } = props;
    const queryClient = useQueryClient();
    // Determine the API URLs based on available endpoints
    const apiURL = apiEndPoint.ALL || `${import.meta.env.VITE_API_END_POINT}/${key}`;
    const getApiURL = apiEndPoint.GET || apiURL;
    const postApiURL = apiEndPoint.POST || apiURL;
    const patchApiURL = apiEndPoint.PATCH || apiURL;
    const deleteApiURL = apiEndPoint.DELETE || apiURL;
    const toggleApiURL = apiEndPoint.TOGGLE || apiURL;
    const putApiURL = apiEndPoint.PUT || apiURL;
    const query = useQuery({
        queryKey: [key, page, limit, sortBy, order, filters, search_term, paginationMethod],
        queryFn: () => getTableData(getToken, key, page, limit, sortBy, order, filters, getApiURL, search_term, paginationMethod, pagination),
    });
    const deleteMutation = useMutation({
        mutationFn: async ({ id }) => {
            await deleteItem(id, deleteApiURL, getToken);
        },
        onSuccess: () => {
            queryClient.invalidateQueries({
                queryKey: [key, page, limit, sortBy, order, filters, search_term],
            });
        },
        onError: error => {
            console.error('Delete item failed:', error);
        },
    });
    const addMutation = useMutation({
        mutationFn: async ({ data }) => {
            const updatedData = handleDetailsSubmit ? await handleDetailsSubmit(data) : data;
            await addItem(updatedData, postApiURL, getToken);
        },
        onSuccess: () => {
            queryClient.invalidateQueries({
                queryKey: [key, page, limit, sortBy, order, filters, search_term],
            });
        },
        onError: error => {
            console.error('Add item failed:', error);
        },
    });
    const updateMutation = useMutation({
        mutationFn: async ({ id, data }) => {
            const updatedData = handleDetailsSubmit ? await handleDetailsSubmit(data) : data;
            const method = editApiMethod === EditApiMethod.PUT ? HttpVerb.PUT : HttpVerb.PATCH;
            const apiURL = editApiMethod === EditApiMethod.PUT ? putApiURL : patchApiURL;
            await updateItem(id, updatedData, apiURL, method, getToken);
        },
        onSuccess: () => {
            queryClient.invalidateQueries({
                queryKey: [key, page, limit, sortBy, order, filters, search_term],
            });
        },
        onError: error => {
            console.error('Update item failed:', error);
        },
    });
    const updateStatusMutation = useMutation({
        mutationFn: async ({ id }) => {
            await updateStatus(id, toggleApiURL, getToken);
        },
        onSuccess: () => {
            queryClient.invalidateQueries({
                queryKey: [key, page, limit, sortBy, order, filters, search_term],
            });
        },
        onError: error => {
            console.error('Update status failed:', error);
        },
    });
    // Function to add new item to the table
    const addNewItem = (data, onSuccess, onError) => {
        addMutation.mutate({ data }, {
            onSuccess: () => {
                if (onSuccess) {
                    onSuccess();
                }
            },
            onError: error => {
                if (onError) {
                    onError(error);
                }
            },
        });
    };
    // Function to update the selected item
    const updateSelectedItem = (id, data, onSuccess, onError) => {
        updateMutation.mutate({ id, data }, {
            onSuccess: () => {
                if (onSuccess) {
                    onSuccess();
                }
            },
            onError: error => {
                if (onError) {
                    onError(error);
                }
            },
        });
    };
    // Function to delete the selected item
    const deleteSelectedItem = (id, onSuccess, onError) => {
        deleteMutation.mutate({ id }, {
            onSuccess: () => {
                if (onSuccess) {
                    onSuccess();
                }
            },
            onError: error => {
                if (onError) {
                    onError(error);
                }
            },
        });
    };
    // Function to update the status of the item
    const updateItemStatus = (id, onSuccess, onError) => {
        updateStatusMutation.mutate({ id }, {
            onSuccess: () => {
                if (onSuccess) {
                    onSuccess();
                }
            },
            onError: error => {
                if (onError) {
                    onError(error);
                }
            },
        });
    };
    return {
        ...query,
        isLoading: query.isLoading,
        isPending: addMutation.isPending || updateMutation.isPending || deleteMutation.isPending,
        addItem: addNewItem,
        deleteItem: deleteSelectedItem,
        updateItem: updateSelectedItem,
        updateStatus: updateItemStatus,
    };
};
export { useTableData };
